# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.__internal__.train namespace.
"""

import sys as _sys

from tensorflow.python.training.checkpoint_management import update_checkpoint_state_internal as update_checkpoint_state
from tensorflow.python.training.experimental.mixed_precision_global_state import is_mixed_precision_graph_rewrite_enabled
from tensorflow.python.training.experimental.mixed_precision_global_state import set_using_mixed_precision_policy
from tensorflow.python.training.moving_averages import assign_moving_average